;;;; "models.scm" utility models for synchronous simulation.	-*-scheme-*-
;;; Copyright (C) 1995, 1996, 1997 Aubrey Jaffer

;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(define (make-fifo length . prototype)
  (set! prototype (if (null? prototype) #f (car prototype)))
  (let ((ara (make-uniform-array prototype (list -2 (+ -1 length)))))
    (uniform-vector-fill! ara 0)
    ara))
(define (fifo:clear fifo)
  (uniform-vector-fill! fifo 0)
  fifo)
(define (fifo:first fifo)
  (logaref fifo (modulo (logaref fifo -2)
			  (+ 1 (cadar (array-shape fifo))))))
(define (fifo:remove-first fifo)
  (if (fifo:empty? fifo) (synch:error 'fifo-underflow fifo))
  (array-set fifo
	     (modulo (+ 1 (logaref fifo -2))
		     (* 2 (+ 1 (cadar (array-shape fifo)))))
	     -2))
(define (fifo:insert-last fifo datum)
  (if (fifo:full? fifo) (synch:error 'fifo-overflow fifo))
  (array-set (array-set fifo datum (modulo (logaref fifo -1)
					   (+ 1 (cadar (array-shape fifo)))))
	     (modulo (+ 1 (logaref fifo -1))
		     (* 2 (+ 1 (cadar (array-shape fifo)))))
	     -1))
(define (fifo:empty? fifo)
  (= (logaref fifo -1) (logaref fifo -2)))
(define (fifo:full? fifo)
  (define len (+ 1 (cadar (array-shape fifo))))
  (= len (modulo (- (logaref fifo -1) (logaref fifo -2))
		 (* 2 len))))
(define (fifo:fullness fifo)
  (modulo (- (logaref fifo -1) (logaref fifo -2))
	  (* 2 (+ 1 (cadar (array-shape fifo))))))
